//set to 1 to use a PIC's internal USB Peripheral
#define __USB_PIC_PERIF__ 1

#include <18F4455.h>

#device adc=16
#fuses HSPLL,NOWDT,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL3,CPUDIV4,VREGEN
#use delay(clock=48000000)

//==============BOOT===========
#define LOADER_END   0xFFF
#build(reset=0x1000, interrupt=0x1008)
#org 0, LOADER_END {}

//============================= 

//Keypad connection: 
#define row0 PIN_B5 
#define row1 PIN_B6 
#define row2 PIN_B7 
#define col0 PIN_B2 
#define col1 PIN_B3 
#define col2 PIN_B4 

//============================= 

/////////////////////////////////////////////////////////////////////////////
//
// Include the CCS USB Libraries.  See the comments at the top of these
// files for more information
//
/////////////////////////////////////////////////////////////////////////////

#include <pic18_usb.h>   //Microchip PIC18Fxx5x hardware layer for usb.c
#include "mJoy.h"   //USB Configuration and Device descriptors for this UBS device
#include <usb.c>        //handles usb setup tokens and get descriptor reports

/////////////////////////////////////////////////////////////////////////////
//
// Configure the demonstration I/O
//
/////////////////////////////////////////////////////////////////////////////
/*
char ConState_j, ConState_p;									// RIGHT, LEFT, DOWN, UP, START, SELECT, B, A
int out_data[3] ={0,0,0};
#define A		0x01
#define B		0x02
#define SELECT	0x04
#define START	0x08
#define UP 		0x40
#define DOWN	0x80
#define LEFT	0x10
#define RIGHT	0x20
*/
int out_data[3] ={0,0,0};
union {
 int state;
 struct 
    {
   int1 A      ;   // "A"      0
   int1 B      ;   // "B"      1
   int1 SELECT ;   // "SELECT" 2
   int1 START  ;   // "START"  3
   int1 UP     ;   // "UP"     4
   int1 DOWN   ;   // "DOWN"   5
   int1 LEFT   ;   // "LEFT"   6   
   int1 RIGHT  ;   // "RIGHT"  7 
    };
}flag;

/////////////////////////////////////////////////////////////////////////////
//
// interrupt function
//
/////////////////////////////////////////////////////////////////////////////
#int_TIMER0
void  TIMER0_isr(void) 
{
output_low(col0); 
output_high(col1); 
output_high(col2);
if (input(row0)==0)flag.SELECT = 1;
if (input(row1)==0)flag.LEFT = 1;
if (input(row2)==0)flag.START = 1; 

output_high(col0); 
output_low(col1); 
output_high(col2);
if (input(row0)==0)flag.UP = 1; 
if (input(row2)==0)flag.DOWN = 1;

output_high(col0); 
output_high(col1); 
output_low(col2); 
if (input(row0)==0)flag.A = 1; 
if (input(row1)==0)flag.RIGHT = 1;
if (input(row2)==0)flag.B = 1;

output_high(col0); 
output_high(col1); 
output_high(col2);
}

/////////////////////////////////////////////////////////////////////////////
//
// usb_debug_task()
//
// When called periodically, displays debugging information over serial
// to display enumeration and connection states.  Also lights LED1 based upon
// enumeration and status.
//
/////////////////////////////////////////////////////////////////////////////
void usb_debug_task(void) 
{
   static int8 last_connected;
   static int8 last_enumerated;
   int8 new_connected;
   int8 new_enumerated;

   new_connected=usb_attached();
   new_enumerated=usb_enumerated();

   last_connected=new_connected;
   last_enumerated=new_enumerated;
}

void main(void) 
{

   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_CLOCK_DIV_2|ADC_TAD_MUL_0);
   setup_psp(PSP_DISABLED);
   setup_spi(SPI_SS_DISABLED);
   setup_wdt(WDT_OFF);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_2);//10.9ms
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_timer_3(T3_DISABLED|T3_DIV_BY_1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);

   enable_interrupts(INT_TIMER0);
   enable_interrupts(GLOBAL);

   port_b_pullups(true); 
   usb_init_cs();

   while (TRUE)
   {  
      usb_task();
      usb_debug_task();



      if (usb_enumerated()) 
      {
	 		out_data[0] = 0;					// Transform the data to match the HID
	 		out_data[1] = 0;                    // This is done this way to match many of
	 		out_data[2] = 0;                    // the older games that require the data to
	 		                                    // come in a specified order
 		
            if(flag.A == 1) // A
	 			out_data[2] |= 0x01;

	 		if(flag.B == 1) // B
	 			out_data[2] |= 0x02;

	 		if(flag.SELECT == 1) // Select
	 			out_data[2] |= 0x04;

	 		if(flag.START == 1) // Start
	 			out_data[2] |= 0x08;
	 		
	 		if(flag.UP == 1) // UP
	 		{
		 		out_data[0] = -127;
		 		out_data[2] |= 0x40;
		 	}
		 	
		 	if(flag.DOWN == 1) // DOWN
		 	{
			 	out_data[0] = 127;
			 	out_data[2] |= 0x80;
			}
	 	
	 		if(flag.LEFT == 1) // LEFT
	 		{
		 		out_data[2] |= 0x10;
		 		out_data[1] = -127;
		 	}
		 	
		 	if(flag.RIGHT == 1) // RIGHT
		 	{
			 	out_data[2] |= 0x20;
			 	out_data[1] = 127;
			}
	 		
	 		usb_put_packet(1, &out_data[0], 3, USB_DTS_TOGGLE);	// Send the data off
            flag.state =0;
	  		delay_ms(20);								// Wait for the next polling event

      }


   }

   
}
